% Figure SM-I 
% replicate the power graph of AMS06 pp732-, but for different rho_{Omega}
% calls the user-defined function CLR_pv.m 
clear all
I=5000; % number of replication
N=100; % sample size
Nsim=100000; %number of simulations when pvals are computed by simulations
k=5; % number of IVs; Values considered, k=2, 5, 10
con=20; % concentration paramter=Pi'Z'ZPi = \lambda,  values considered: 5, 20

omega_1=1;omega_2=1; 
omega_12=0.0; %  This is rho_{Omega} it took 0.5 and 0.95 in AMS06
Omega=[omega_1^2,omega_12;omega_12,omega_2^2];%reduced form covariance matrix
beta0=0; % following AMS06, fixed beta_0, different from AMY18
b_0=[1,-beta0]'; a_0=[beta0,1]';

alpha=0.05;

sc=6/sqrt(con); % following AMS06, the x-axis is beta_star*sqrt(con), and the range is [-6,6]
step=0.05; l=round(sc/step);
Beta_star=[-l*step:step:l*step]'; %alternative values 

Beta_star=Beta_star+0.00001*(Beta_star==1); %with beta_star=1, c_beta_2star=infinity
Beta_star=Beta_star+0.00001*(Beta_star==0);
Beta_star=Beta_star+0.00001*(Beta_star==2); %with beta_0=2, c_beta_2star=infinity for omega_12=0.5
Lg=length(Beta_star);
AR=zeros(I,Lg);
LR=zeros(I,Lg);
KLM=zeros(I,Lg);
AMSPOIS2=zeros(I,Lg);
p_AR=zeros(I,Lg);
p_CLR=zeros(I,Lg);
p_KLM=zeros(I,Lg);


p_AMSPOIS2_sim=zeros(I,Lg);

rej_AR=zeros(I,Lg);
rej_CLR=zeros(I,Lg);
rej_KLM=zeros(I,Lg);
rej_AMSPOIS2_sim=zeros(I,Lg);
rng(0)
randseedoffset=81474;

tic
%for AMSPOIS2: C=c_beta, D=d_beta, Ksi=ksi_beta(Q);  In tables POIS2
AMS_psi = @(C,D,Ksi,lambda) exp(-lambda*(C^2+D^2)/2) ...
    *(lambda*Ksi).^(-k/4+0.5)...
    .*besseli(k/2-1,sqrt(lambda*Ksi));
AMS_psi2=@(D,q_t) exp(-con*D^2/2) ...
    *(con*D^2*q_t).^(-k/4+0.5) ...
    .*besseli(k/2-1,sqrt(con*D^2*q_t));
%---------------------------------

%for simulated p-vals
SS=normrnd(0,1,Nsim,k);
SQ_s=sum(SS.*SS,2);


parfor i=1:I
    
    rng(randseedoffset+i, 'twister'); %to generate the same random sequences
    
    SS0=randn(k,1);
    TT0=randn(k,1);

    for j=1:Lg;
        beta_star=Beta_star(j);
        b=[1,-beta_star]';
        d_beta_star=(b'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
        c_beta_star=(beta_star-beta0)*(b_0'*Omega*b_0)^(-1/2);
        e_1=[1; zeros(k-1,1)];        
        mu=sqrt(con)*e_1;
        S=SS0+c_beta_star*mu;
        T=TT0+d_beta_star*mu;
        Q_s=S'*S;
        Q_st=S'*T;
        Q_t=T'*T;
        S_2=Q_st/(Q_s*Q_t)^(1/2);
        
        AR(i,j)=Q_s;
        KLM(i,j)=Q_st^2/Q_t;
        LR(i,j)=0.5*(Q_s-Q_t+((Q_s+Q_t)^2-4*Q_s*Q_t*(1-S_2^2))^(1/2));
        
        % compute AMS POIS2 (with exact beta0)
        ksi_beta_star=c_beta_star^2*Q_s+2*c_beta_star*d_beta_star*Q_st ...
            +d_beta_star^2*Q_t;
        
        r_beta0=[1 0]*inv(Omega)*a_0*(a_0'*inv(Omega)*a_0)^(-1/2);
        d_beta0=(b_0'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
        beta_2star=beta0-d_beta0*(beta_star-beta0) ...
            /(d_beta0+2*r_beta0*(beta_star-beta0));
        lambda2=con*(d_beta0+2*r_beta0*(beta_star-beta0))^2/d_beta0^2;
        
        b2=[1,-beta_2star]';
        d_beta_2star=(b2'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
        c_beta_2star=(beta_2star-beta0)*(b_0'*Omega*b_0)^(-1/2);
        ksi_beta_2star=c_beta_2star^2*Q_s+2*c_beta_2star*d_beta_2star*Q_st ...
            +d_beta_2star^2*Q_t;
        
        
        AMSPOIS2(i,j)=(AMS_psi(c_beta_star,d_beta_star,ksi_beta_star,con) ...
            +AMS_psi(c_beta_2star,d_beta_2star,ksi_beta_2star,lambda2) )...
            /AMS_psi2(d_beta_star,Q_t)/2;
        %---------------------------------
        % p-values
        p_AR(i,j)=1-chi2cdf(AR(i,j),k);
        p_KLM(i,j)=1-chi2cdf(KLM(i,j),1);
        p_CLR(i,j)=CLR_pv(LR(i,j),Q_t,k); %by num integr
        % comptue rejection frequency
        rej_AR(i,j)=p_AR(i,j)<alpha;
        rej_KLM(i,j)=p_KLM(i,j)<alpha;
        rej_CLR(i,j)=p_CLR(i,j)<alpha;
        %for simulated p-vals
        SQ_st=SS*T;
        %simulated p-vals for AMSPOIS2------------------------
        %p_AMSPOIS2_sim(i,j)=AMSPOIS2_pv_sim(AMSPOIS2(i,j),T,k,con,c_beta_star,d_beta_star,c_beta_2star,d_beta_2star,lambda2,Nsim);
        S_ksi_beta_star=c_beta_star^2*SQ_s+2*c_beta_star*d_beta_star*SQ_st ...
            +d_beta_star^2*Q_t;
        S_ksi_beta_2star=c_beta_2star^2*SQ_s+2*c_beta_2star*d_beta_2star*SQ_st ...
            +d_beta_2star^2*Q_t;
        S_AMSPOIS2=(AMS_psi(c_beta_star,d_beta_star,S_ksi_beta_star,con) ...
            +AMS_psi(c_beta_2star,d_beta_2star,S_ksi_beta_2star,lambda2) )...
            /AMS_psi2(d_beta_star,Q_t)/2;
        p_AMSPOIS2_sim(i,j)=1-sum(S_AMSPOIS2<AMSPOIS2(i,j))/Nsim;
        rej_AMSPOIS2_sim(i,j)=p_AMSPOIS2_sim(i,j)<alpha;
        %------------------------------------------------------------
        
    end
        
    
    
end
Rej_AR=mean(rej_AR);
Rej_KLM=mean(rej_KLM);
Rej_CLR=mean(rej_CLR);
Rej_AMSPOIS2_sim=mean(rej_AMSPOIS2_sim); %This is POIS2

toc
figure
plot(Beta_star*sqrt(con),Rej_AMSPOIS2_sim,'k-', 'LineWidth',2)
hold on
plot(Beta_star*sqrt(con),Rej_CLR,'k--')
plot(Beta_star*sqrt(con), Rej_AR,'k-')
plot(Beta_star*sqrt(con), Rej_KLM,'k:')
hold off
axis([-6,6,0,1])
legend('POIS2','CLR','AR','LM','location','best')
xlabel('$\beta_*\sqrt{\lambda}$','Interpreter','latex')
ylabel('power')
%title('$k=10$, $\rho_\Omega=0$, $\lambda=20$','Interpreter','latex')

%save graph_AMS_rho0_lambda20_k5

toc/60
