% For Table I, Table SM-I and Table SM-VI
% Probabilities of Infinite-length confidence intervals for 
% AR, CLR, POIS_infty, POIS2_infty, and CLR_2
% calls user-defined functions BD_pv.m and CLR_pv.m
clear all

I=50000; % number of replication
Nsim=100000; %number of simulations when pvals are computed by simulations

rho=.0; %correlation between u and v,
        % takes values 0, .3, .5, .7, .9.

fprintf('$\\rho='),fprintf('%8.2f $, \n',rho);
fprintf('k & $\\lambda$  &      AR &     CLR & CLR2_n1 &  CLR2_n2 & CLR2_n3 & POIS  &    POIS2  &    $AR-CLR$ & $SD_{AR-CLR}$& $AR--POIS2$ & $CLR--POIS2$ & $SD_{CLR-POIS2}$& $POIS--POIS2$ \\\\ \\hline \n');

for k=[2 5 10 20 40] % k is number of IVs;

if k==2
  Con=[1 3 5 7 10 15 20]; % concentration parameters \lambda
elseif k==5
  Con=[1 3 5 7 10 12 15 20 25];
elseif k==10
  Con=[1 5 10 15 17 20 25 30];
elseif k==20
  Con=[1 5 10 15 20 25 30 40];
elseif k==40
  Con=[1 5 10 15 20 30 40 60];
end 

Lcon=length(Con);

n1=100; n2=500; n3=1000; % n for computing CLR2
alpha=0.05;
Beta0=[-10000, 10000]; %set the null $\beta_0$ to be very large
Lg=length(Beta0);
beta_star=0;
omega_1=1;omega_2=1;

AR=zeros(I,Lg);
LR=zeros(I,Lg);
LR21=zeros(I,Lg);LR22=zeros(I,Lg);LR23=zeros(I,Lg);
KLM=zeros(I,Lg);
BD=zeros(I,Lg);
POIS2=zeros(I,Lg);
p_AR=zeros(I,Lg);
p_CLR=zeros(I,Lg);
p_CLR_sim=zeros(I,Lg);
p_CLR21_sim=zeros(I,Lg);p_CLR22_sim=zeros(I,Lg);p_CLR23_sim=zeros(I,Lg);
p_KLM=zeros(I,Lg);
p_BD=zeros(I,Lg);
p_BD_sim=zeros(I,Lg);
p_POIS2_sim=zeros(I,Lg);

rej_AR=zeros(I,Lg);
rej_CLR=zeros(I,Lg);
rej_CLR_sim=zeros(I,Lg);
rej_CLR21_sim=zeros(I,Lg); rej_CLR22_sim=zeros(I,Lg); rej_CLR23_sim=zeros(I,Lg);
rej_KLM=zeros(I,Lg);
rej_BD=zeros(I,Lg);
rej_POIS2_sim=zeros(I,Lg);

tic
for t=1:Lcon; % the loop for different concentration parameter \lambda
con=Con(t);
r_uv=rho/sqrt((1-rho^2));
lambda_v=con/omega_2;
Omega=[omega_1^2,rho;rho,omega_2^2];%reduced form covariance matrix
sigma_1=sqrt(omega_1^2-2*rho*beta_star+omega_2^2*beta_star^2); sigma_12=rho-omega_2^2*beta_star;sigma_2=omega_2;
Sigma=[sigma_1^2,sigma_12;sigma_12,sigma_2^2];
rng(0)
randseedoffset=81474;

%for POIS2_{infty}
ksi=@(q_s,q_st,q_t,r) q_s+2*r*q_st+r^2*q_t;
psi=@(q_s,q_st,q_t,r) exp(-lambda_v*(1+r^2)/2)...
    *(lambda_v*ksi(q_s,q_st,q_t,r)).^(0.5-k/4)...
    .*besseli(k/2-1,sqrt(lambda_v*ksi(q_s,q_st,q_t,r)));
psi2=@(q_t,r) exp(-lambda_v*r^2/2)*(lambda_v*r^2*q_t).^(0.5-k/4)...
    .*besseli(k/2-1,sqrt(lambda_v*r^2*q_t));
%for simulated p-vals
SS=normrnd(0,1,Nsim,k);
SQ_s=sum(SS.*SS,2);

parfor i=1:I  % the loop for simulation replications
    rng(randseedoffset+i, 'twister'); 
    SS0=randn(k,1);
    TT0=randn(k,1);
    for j=1:Lg
    beta0=Beta0(j);
    b_0=[1,-beta0]'; a_0=[beta0,1]';
    % compute c, d and mu in Lemma 2 of AMS06
    b=[1,-beta_star]';
    d_beta_star=(b'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
    c_beta_star=(beta_star-beta0)*(b_0'*Omega*b_0)^(-1/2);
    e_1=[1; zeros(k-1,1)];
    
    miu=sqrt(con)*e_1;
    S=SS0+c_beta_star*miu;
    T=TT0+d_beta_star*miu;

    Q_s=S'*S;
    Q_st=S'*T;
    Q_t=T'*T;
    S_2=Q_st/(Q_s*Q_t)^(1/2);
        
    AR(i,j)=Q_s;
    KLM(i,j)=Q_st^2/Q_t;
    LR(i,j)=0.5*(Q_s-Q_t+((Q_s+Q_t)^2-4*Q_s*Q_t*(1-S_2^2))^(1/2));
    BD(i,j)=Q_s+2*rho/sqrt(1-rho^2)*Q_st;
    
    % compute LR2
    LR21(i,j)=n1*log(1+Q_s/(n1-k))-n1*log(1+(Q_s-LR(i,j))/(n1-k));
    LR22(i,j)=n2*log(1+Q_s/(n2-k))-n2*log(1+(Q_s-LR(i,j))/(n2-k));
    LR23(i,j)=n3*log(1+Q_s/(n3-k))-n3*log(1+(Q_s-LR(i,j))/(n3-k));
    
    %compute POIS2 (as beta0->infinity)
    if r_uv~= 0
       POIS2(i,j)=(psi(Q_s,Q_st,Q_t,r_uv)+psi(Q_s,Q_st,Q_t,-r_uv))/2 ...
       /psi2(Q_t,r_uv);
    else
       POIS2(i,j)=(psi(Q_s,Q_st,Q_t,r_uv)+psi(Q_s,Q_st,Q_t,-r_uv))/2 ...
       /(0.5^(k/2-1)/gamma(k/2));
    end
    % p-values
    p_AR(i,j)=1-chi2cdf(AR(i,j),k);
    p_KLM(i,j)=1-chi2cdf(KLM(i,j),1);
    p_CLR(i,j)=CLR_pv(LR(i,j),Q_t,k); %by num integr
    p_BD(i,j)=BD_pv(BD(i,j),Q_t,k,rho); %by num integr
        
   %rejection frequency
   rej_AR(i,j)=p_AR(i,j)<alpha;
   rej_KLM(i,j)=p_KLM(i,j)<alpha;
   rej_CLR(i,j)=p_CLR(i,j)<alpha;
   rej_BD(i,j)=p_BD(i,j)<alpha;
        
   %for simulated p-vals
   SQ_st=SS*T;
        
   %simulated p-vals for CLR2
   S_LR=0.5*(SQ_s-Q_t+sqrt((SQ_s-Q_t).^2+4*SQ_st.^2));
   p_CLR_sim(i,j)=1-sum(S_LR<LR(i,j))/Nsim;
   rej_CLR_sim(i,j)=p_CLR_sim(i,j)<alpha;
   
   S_LR21=n1*log(1+SQ_s/(n1-k))-n1*log(1+(SQ_s-S_LR)/(n1-k));
   p_CLR21_sim(i,j)=1-sum(S_LR21<LR21(i,j))/Nsim;
   rej_CLR21_sim(i,j)=p_CLR21_sim(i,j)<alpha;
   
   S_LR22=n2*log(1+SQ_s/(n2-k))-n2*log(1+(SQ_s-S_LR)/(n2-k));
   p_CLR22_sim(i,j)=1-sum(S_LR22<LR22(i,j))/Nsim;
   rej_CLR22_sim(i,j)=p_CLR22_sim(i,j)<alpha;
   
   S_LR23=n3*log(1+SQ_s/(n3-k))-n3*log(1+(SQ_s-S_LR)/(n3-k));
   p_CLR23_sim(i,j)=1-sum(S_LR23<LR23(i,j))/Nsim;
   rej_CLR23_sim(i,j)=p_CLR23_sim(i,j)<alpha;
   
   %simulated p-vals for POIS2_inf
   if r_uv~=0
      S_POIS2=(psi(SQ_s,SQ_st,Q_t,r_uv)+psi(SQ_s,SQ_st,Q_t,-r_uv))/2 ...
       /psi2(Q_t,r_uv);
   else
      S_POIS2=(psi(SQ_s,SQ_st,Q_t,r_uv)+psi(SQ_s,SQ_st,Q_t,-r_uv))/2 ...
      /(0.5^(k/2-1)/gamma(k/2));
   end
      p_POIS2_sim(i,j)=1-sum(S_POIS2<POIS2(i,j))/Nsim;
      rej_POIS2_sim(i,j)=p_POIS2_sim(i,j)<alpha;
   end
        
end

% the probability that the AR confidence interval has infinit length.
Infty_CI_AR_lr=mean(1-rej_AR); 
Infty_CI_KLM_lr=mean(1-rej_KLM);
Infty_CI_CLR_lr=mean(1-rej_CLR);
Infty_CI_CLRs_lr=mean(1-rej_CLR_sim);
Infty_CI_CLR21_lr=mean(1-rej_CLR21_sim);Infty_CI_CLR22_lr=mean(1-rej_CLR22_sim);Infty_CI_CLR23_lr=mean(1-rej_CLR23_sim);
Infty_CI_POIS_lr=mean(1-rej_BD);
Infty_CI_POIS2_lr=mean(1-rej_POIS2_sim);

Infty_CI_AR = Infty_CI_AR_lr(2); 
Infty_CI_KLM = Infty_CI_KLM_lr(2);
Infty_CI_CLR = Infty_CI_CLR_lr(2);
Infty_CI_CLRs = Infty_CI_CLRs_lr(2);
Infty_CI_CLR2 = [Infty_CI_CLR21_lr(2),Infty_CI_CLR22_lr(2),Infty_CI_CLR23_lr(2)];
Infty_CI_POIS = Infty_CI_POIS_lr(2);
Infty_CI_POIS2 = Infty_CI_POIS2_lr(2);

sd2=sqrt(var(rej_AR-rej_CLR)/I);
%implied correlation
Rho0=(Omega(1,2)-Beta0*Omega(2,2))./(Omega(1,1)-2*Beta0*Omega(1,2)+Beta0.^2*Omega(2,2)).^(0.5);
fprintf([sprintf('%3.0f & %3.0f &',k, con) ...
      strrep(sprintf('%.3f & %.3f & %.3f & %.3f & %.3f & %.3f & %.3f & %.3f & %.4f & %.3f & %.3f & %.4f & %.3f\\\\\\\\ \n', ...
      Infty_CI_AR, Infty_CI_CLR, Infty_CI_CLR2(1),Infty_CI_CLR2(2), Infty_CI_CLR2(3),Infty_CI_POIS, Infty_CI_POIS2, Infty_CI_CLR-Infty_CI_AR, sd2(2), Infty_CI_AR-Infty_CI_POIS2, Infty_CI_CLR-Infty_CI_POIS2, sd1(2), Infty_CI_POIS-Infty_CI_POIS2),'0.','.')]);
% save the results
%filename=sprintf('Prob_infty_CI2_%dk%dc%d.mat',rho*10,k,con);
%save(filename,'I','Nsim','n1','n2','n3','rho','k','con','rej_AR','rej_KLM','rej_CLR','rej_CLR_sim','rej_CLR21_sim','rej_CLR22_sim','rej_CLR23_sim','rej_BD','rej_POIS2_sim');
end
toc
end


