% For Table II, Table SM-II
%Computes the power of POIS2 and CLR, and picks the value of beta0 with
%the max difference
% calls the user-defined function CLR_pv.m

clear all

feature('accel','on')
I=5000; % number of replication
N=100; % sample size
Nsim=100000; %number of simulations when pvals are computed by simulations
%filename='~/matlab/linear_weak_iv/output0_10.mat';
rho=0; %correlation between u and v, take values 0, .3, .5, .7, .9.
k=10; % number of IVs;
Lambda=[1 3 5 7 10 15 20]; % concentration paramter=Pi'Z'ZPi = lambda in the notes
%Lambda=[22 25]; large lambda for k=20, 40
%Lambda=[0.7 0.8 0.9 1]; small lambda for k=2, 5 with rho=.9.

beta_star=0;
omega_1=1;omega_2=1;
r_uv=rho/sqrt((1-rho^2));
Omega=[omega_1^2,rho;rho,omega_2^2];%reduced form covariance matrix
sigma_1=sqrt(omega_1^2-2*rho*beta_star+omega_2^2*beta_star^2); sigma_12=rho-omega_2^2*beta_star;sigma_2=omega_2;
Sigma=[sigma_1^2,sigma_12;sigma_12,sigma_2^2];

alpha=0.05;
Beta0=[-10000, -1000, -100, -50, -10, -7.5, -5, [-4:0.25:-0.25],[0.25:0.25:4],5,7.5,10,50,100,1000,10000]; %values under the null
Beta0=Beta0+0.00001*(Beta0==1); %with beta_0=1, c_beta_2star=infinity
Beta0=Beta0+0.00001*(Beta0==2); %with beta_0=2, c_beta_2star=infinity for rho=0.5
Beta0=Beta0-0.00001*(Beta0==-1); %there is a problem when rho=0

Lg=length(Beta0);
Lc=length(Lambda);


LR=zeros(I,Lg);
AMSPOIS2=zeros(I,Lg);
p_CLR=zeros(I,Lg);
p_AMSPOIS2_sim=zeros(I,Lg);
rej_CLR=zeros(Lc,Lg,I);
rej_AMSPOIS2_sim=zeros(Lc,Lg,I);

randseedoffset=81474; % was used in all simulations
tic
%for AMSPOIS2: C=c_beta, D=d_beta, Ksi=ksi_beta(Q);  In tables POIS2
AMS_psi = @(C,D,Ksi,lambda) exp(-lambda*(C^2+D^2)/2) ...
    *(lambda*Ksi).^(-k/4+0.5)...
    .*besseli(k/2-1,sqrt(lambda*Ksi));
AMS_psi2=@(D,q_t,lambda) exp(-lambda*D^2/2) ...
    *(lambda*D^2*q_t).^(-k/4+0.5) ...
    .*besseli(k/2-1,sqrt(lambda*D^2*q_t));
%---------------------------------
%for simulated p-vals
SS=normrnd(0,1,Nsim,k);
SQ_s=sum(SS.*SS,2);
parfor i=1:I
    rng(randseedoffset+i, 'twister'); %to generate the same random sequences
    Z = randn(N,k); % Z are instruments
    epsilon= randn(N,2);
    epsilon1=epsilon(:,1);
    epsilon2 = epsilon(:,2);
    U = epsilon1;
    V_2 = sigma_12*epsilon1 + sqrt(1-sigma_12^2)*epsilon2;
    for c=1:Lc
        con=Lambda(c);
        %lambda_v=con/omega_2;
    Pi=ones(k,1)*sqrt(con/sum(sum(Z'*Z)));
    y_2 = Z*Pi + V_2;
    y_1 = y_2*beta_star+U;
    Y=[y_1,y_2]; V_1=U+V_2*beta_star;V=[V_1, V_2];
    
    for j=1:Lg;
        beta0=Beta0(j);
        b_0=[1,-beta0]'; a_0=[beta0,1]';
        S=(Z'*Z)^(-1/2)*Z'*Y*b_0/(b_0'*Omega*b_0)^(1/2);
        T=(Z'*Z)^(-1/2)*Z'*Y/Omega*a_0/(a_0'/Omega*a_0)^(1/2);
        Q_s=S'*S;
        Q_st=S'*T;
        Q_t=T'*T;
        S_2=Q_st/(Q_s*Q_t)^(1/2);
        LR(i,j)=0.5*(Q_s-Q_t+((Q_s+Q_t)^2-4*Q_s*Q_t*(1-S_2^2))^(1/2));
        % compute AMS POIS2 (with exact beta0)
        b=[1,-beta_star]';
        d_beta_star=(b'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
        c_beta_star=(beta_star-beta0)*(b_0'*Omega*b_0)^(-1/2);
        ksi_beta_star=c_beta_star^2*Q_s+2*c_beta_star*d_beta_star*Q_st ...
            +d_beta_star^2*Q_t;
        
        r_beta0=[1 0]*inv(Omega)*a_0*(a_0'*inv(Omega)*a_0)^(-1/2);
        d_beta0=(b_0'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
        beta_2star=beta0-d_beta0*(beta_star-beta0) ...
            /(d_beta0+2*r_beta0*(beta_star-beta0));
        lambda2=con*(d_beta0+2*r_beta0*(beta_star-beta0))^2/d_beta0^2;
        
        b2=[1,-beta_2star]';
        d_beta_2star=(b2'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
        c_beta_2star=(beta_2star-beta0)*(b_0'*Omega*b_0)^(-1/2);
        ksi_beta_2star=c_beta_2star^2*Q_s+2*c_beta_2star*d_beta_2star*Q_st ...
            +d_beta_2star^2*Q_t;
        
        
        AMSPOIS2(i,j)=(AMS_psi(c_beta_star,d_beta_star,ksi_beta_star,con) ...
            +AMS_psi(c_beta_2star,d_beta_2star,ksi_beta_2star,lambda2) )...
            /AMS_psi2(d_beta_star,Q_t,con)/2;
        %---------------------------------
        % p-values
        p_CLR(i,j)=CLR_pv(LR(i,j),Q_t,k); %by num integr
        % comptue rejection frequency
        rej_CLR(c,j,i)=p_CLR(i,j)<alpha;
        %for simulated p-vals
        SQ_st=SS*T;
        %simulated p-vals for AMSPOIS2------------------------
        %p_AMSPOIS2_sim(i,j)=AMSPOIS2_pv_sim(AMSPOIS2(i,j),T,k,con,c_beta_star,d_beta_star,c_beta_2star,d_beta_2star,lambda2,Nsim);
        S_ksi_beta_star=c_beta_star^2*SQ_s+2*c_beta_star*d_beta_star*SQ_st ...
            +d_beta_star^2*Q_t;
        S_ksi_beta_2star=c_beta_2star^2*SQ_s+2*c_beta_2star*d_beta_2star*SQ_st ...
            +d_beta_2star^2*Q_t;
        S_AMSPOIS2=(AMS_psi(c_beta_star,d_beta_star,S_ksi_beta_star,con) ...
            +AMS_psi(c_beta_2star,d_beta_2star,S_ksi_beta_2star,lambda2) )...
            /AMS_psi2(d_beta_star,Q_t,con)/2;
        p_AMSPOIS2_sim(i,j)=1-sum(S_AMSPOIS2<AMSPOIS2(i,j))/Nsim;
        rej_AMSPOIS2_sim(c,j,i)=p_AMSPOIS2_sim(i,j)<alpha;
        %------------------------------------------------------------
        
    end
    end    
    
    
end

Rej_CLR=mean(rej_CLR,3);
Rej_AMSPOIS2_sim=mean(rej_AMSPOIS2_sim,3); %This is POIS2

Diff=Rej_AMSPOIS2_sim-Rej_CLR;
[M indx]=max(Diff,[],2);

toc
%implied correlation
Rho0=(Omega(1,2)-Beta0*Omega(2,2))./(Omega(1,1)-2*Beta0*Omega(1,2)+Beta0.^2*Omega(2,2)).^(0.5);
disp(' ')
fprintf('\\begin{table} \n');
fprintf('\\centering  \n');

fprintf('\\caption{ \n');
fprintf('$\\rho_{uv}='),fprintf('%8.2f $, \n',rho);
fprintf('} \n');

fprintf(' \\begin{tabular}{cc.STT}\\hline \n');
fprintf('$k$ & $\\lambda$ & \\multicolumn{1}{c}{$\\beta_{0,\\textrm{max}}$}  &\\multicolumn{1}{c}{$\\rho_{uv,0}$} &     \\multicolumn{1}{c}{POIS2} &      \\multicolumn{1}{c}{POIS2--CLR} \\\\ \\hline \n');

for j=1:Lc
    fprintf([ ...
        sprintf('%2.0f & ',k) ...
        sprintf('%2.0f & ',Lambda(j)) ...
        sprintf('%8.2f & ',Beta0(indx(j))) ...
        strrep(sprintf('%.2f & ',Rho0(indx(j))),'0.','.') ...
        strrep(sprintf('%.3f & ',Rej_AMSPOIS2_sim(j,indx(j))),'0.','.') ...
        strrep(sprintf('%.3f ',M(j)),'0.','.') ...
        ' \\\\ \n' ...
    ]);
end

fprintf('\\hline \n')
fprintf('\\end{tabular}\n')
fprintf('\\end{table}\n')
disp(' ')

toc/3600


%save(filename,'Beta0','Lambda','k','rho','Rej_AMSPOIS2_sim','Rej_CLR','indx','M','Diff','rej_CLR','rej_AMSPOIS2_sim');
