% For Table III, Table IV and Table SM-V
%Average(over \lambda) power difference WAP2-CLR, for rho=.0, .3, .5, .7,
%.9.
% calls user-defined function CLR_pv.m
clear all

I=1000; % number of replication
N=100; % sample size
Nsim=100000; %number of simulations when pvals are computed by simulations
k=5; % number of IVs; other values considered: k=2, 10, 20, 40
Con=[2.5:2.5:90]'; % \lambda, concentration parameters
Lc=length(Con);
beta_star=0; % true beta
omega_1=1;omega_2=1; 

for rho=[0,0.3,0.5,0.7,0.9]; %correlation between u and v
 r_uv=rho/sqrt((1-rho^2));
 Omega=[omega_1^2,rho;rho,omega_2^2];%reduced form covariance matrix
 sigma_1=sqrt(omega_1^2-2*rho*beta_star+omega_2^2*beta_star^2); sigma_12=rho-omega_2^2*beta_star;sigma_2=omega_2;
 Sigma=[sigma_1^2,sigma_12;sigma_12,sigma_2^2];
 alpha=0.05;
 Beta0=[-10000, -1000,-100,-50,-10,-7.5,-5.0,-4:0.25:-0.25,0.25:0.25:4,5.0,7.5,10,50,100,1000,10000];
 %values under the null
 Beta0=Beta0+0.00001*(Beta0==1); %with beta_0=0, c_beta_2star=infinity
 Beta0=Beta0+0.00001*(Beta0==2); %with beta_0=0, c_beta_2star=infinity for rho=0.5
 Beta0=Beta0-0.00001*(Beta0==-1);
 Lg=length(Beta0);
 AR=zeros(I,Lg,Lc);
 LR=zeros(I,Lg,Lc);
 KLM=zeros(I,Lg,Lc);
 POIS2=zeros(I,Lg,Lc);
 POIS2_2=zeros(I,Lg,Lc);
 p_AR=zeros(I,Lg,Lc);
 p_CLR=zeros(I,Lg,Lc);
 p_KLM=zeros(I,Lg,Lc);
 p_POIS2=zeros(I,Lg,Lc);
 p_POIS2_2=zeros(I,Lg,Lc);
 rej_AR=zeros(I,Lg,Lc);
 rej_CLR=zeros(I,Lg,Lc);
 rej_KLM=zeros(I,Lg,Lc);
 rej_POIS2=zeros(I,Lg,Lc);
 rej_POIS2_2=zeros(I,Lg,Lc);
 
 rng(0)
 randseedoffset=81474;
 
 %for AMSPOIS2: C=c_beta, D=d_beta, Ksi=ksi_beta(Q); 
 psi = @(C,D,Ksi,lambda) exp(-lambda.*(C.^2+D.^2)/2).*(lambda.*Ksi).^(-k/4+0.5)...
    .*besseli(k/2-1,sqrt(lambda.*Ksi));
 psi2=@(D,q_t,lambda) exp(-lambda.*D.^2/2).*(lambda.*D.^2.*q_t).^(-k/4+0.5) ...
    .*besseli(k/2-1,sqrt(lambda.*D.^2.*q_t));
%---------------------------------

 %for simulated p-vals
 SS=normrnd(0,1,Nsim,k);
 SQ_s=sum(SS.*SS,2);

 tic 
 parfor i=1:I
    
    rng(randseedoffset+i, 'twister'); %to generate the same random sequences
    ZZ = randn(N,k,Lc); % instruments
    epsilon= randn(N,2,Lc);
    for j=1:Lg;
      beta0=Beta0(j);
      b_0=[1,-beta0]'; a_0=[beta0,1]';
      % compute the other point
      b=[1,-beta_star]';
      d_beta_star=(b'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
      c_beta_star=(beta_star-beta0)*(b_0'*Omega*b_0)^(-1/2);
      r_beta0=[1 0]/(Omega)*a_0*(a_0'/(Omega)*a_0)^(-1/2);
      d_beta0=(b_0'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
      beta_2star=beta0-d_beta0*(beta_star-beta0)/(d_beta0+2*r_beta0*(beta_star-beta0));
      Con2=Con*(d_beta0+2*r_beta0*(beta_star-beta0))^2/d_beta0^2;
      b2=[1,-beta_2star]';
      d_beta_2star=(b2'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
      c_beta_2star=(beta_2star-beta0)*(b_0'*Omega*b_0)^(-1/2);
      
      for p=1:Lc
        con=Con(p);
        epsilon1=epsilon(:,1,p);
        epsilon2 = epsilon(:,2,p);
        Z=ZZ(:,:,p);
        U = epsilon1;
        V_2 = sigma_12*epsilon1 + sqrt(1-sigma_12^2)*epsilon2;
        Pi=ones(k,1)*sqrt(con/sum(sum(Z'*Z)));
        y_2 = Z*Pi + V_2;
        y_1 = y_2*beta_star+U;
        Y=[y_1,y_2]; V_1=U+V_2*beta_star;V=[V_1, V_2];
        
        S=(Z'*Z)^(-1/2)*Z'*Y*b_0/(b_0'*Omega*b_0)^(1/2);
        T=(Z'*Z)^(-1/2)*Z'*Y/Omega*a_0/(a_0'/Omega*a_0)^(1/2);
        Q_s=S'*S;
        Q_st=S'*T;
        Q_t=T'*T;
        S_2=Q_st/(Q_s*Q_t)^(1/2);
        
        AR(i,j,p)=Q_s;
        KLM(i,j,p)=Q_st^2/Q_t;
        LR(i,j,p)=0.5*(Q_s-Q_t+((Q_s+Q_t)^2-4*Q_s*Q_t*(1-S_2^2))^(1/2));
        % compute POIS2 
        %POIS2 under the data generated by (beta_star, con)
        ksi_beta_star=c_beta_star^2*Q_s+2*c_beta_star*d_beta_star*Q_st ...
            +d_beta_star^2*Q_t;
        ksi_beta_2star=c_beta_2star^2*Q_s+2*c_beta_2star*d_beta_2star*Q_st ...
            +d_beta_2star^2*Q_t;
        POIS2(i,j,p)=(mean(psi(c_beta_star,d_beta_star,ksi_beta_star,Con)) ...
            +mean(psi(c_beta_2star,d_beta_2star,ksi_beta_2star,Con2)))...
            /mean(psi2(d_beta_star,Q_t,Con))/2;
        %POIS2 under the data generated by (beta_2star, con2)
        ksi_beta_star_2=c_beta_star^2*Q_s+2*c_beta_star*d_beta_star*(-Q_st) ...
            +d_beta_star^2*Q_t;
        ksi_beta_2star_2=c_beta_2star^2*Q_s+2*c_beta_2star*d_beta_2star*(-Q_st) ...
            +d_beta_2star^2*Q_t;
        POIS2_2(i,j,p)=(mean(psi(c_beta_star,d_beta_star,ksi_beta_star_2,Con)) ...
            +mean(psi(c_beta_2star,d_beta_2star,ksi_beta_2star_2,Con2)) )...
            /mean(psi2(d_beta_star,Q_t,Con))/2;
        %-------------------------------
        % p-values
        p_AR(i,j,p)=1-chi2cdf(AR(i,j,p),k);
        p_KLM(i,j,p)=1-chi2cdf(KLM(i,j,p),1);
        p_CLR(i,j,p)=CLR_pv(LR(i,j,p),Q_t,k); %by num integr
        % comptue rejection frequency
        rej_AR(i,j,p)=p_AR(i,j,p)<alpha;
        rej_KLM(i,j,p)=p_KLM(i,j,p)<alpha;
        rej_CLR(i,j,p)=p_CLR(i,j,p)<alpha;
        %for simulated p-vals
        SQ_st=SS*T;
        %simulated p-vals for AMSPOIS2------------------------
        S_POIS2_nu=zeros(Nsim,Lc);
        S_POIS2_de=zeros(Nsim,Lc);
        for t=1:Lc
        S_ksi_beta_star=c_beta_star^2*SQ_s+2*c_beta_star*d_beta_star*SQ_st ...
            +d_beta_star^2*Q_t;
        S_ksi_beta_2star=c_beta_2star^2*SQ_s+2*c_beta_2star*d_beta_2star*SQ_st ...
            +d_beta_2star^2*Q_t;
        S_POIS2_nu(:,t)=(psi(c_beta_star,d_beta_star,S_ksi_beta_star,Con(t))...
            +psi(c_beta_2star,d_beta_2star,S_ksi_beta_2star,Con2(t)));
        S_POIS2_de(:,t)=psi2(d_beta_star,Q_t,Con(t));
        end
        S_POIS2=mean(S_POIS2_nu,2)./mean(S_POIS2_de,2)/2;
        p_POIS2(i,j,p)=1-sum(S_POIS2<POIS2(i,j,p))/Nsim; % under the data generated by (beta_star, con)
        rej_POIS2(i,j,p)=p_POIS2(i,j,p)<alpha;
        p_POIS2_2(i,j,p)=1-sum(S_POIS2<POIS2_2(i,j,p))/Nsim; % under the data generated by (beta_2star, con2)
        rej_POIS2_2(i,j,p)=p_POIS2_2(i,j,p)<alpha; 
        %------------------------------------------------------------
        
      end
        
    end
    
 end
toc
Rej_AR=mean(mean(rej_AR),3);
Rej_KLM=mean(mean(rej_KLM),3);
Rej_CLR=mean(mean(rej_CLR),3);
Rej_POIS2=(mean(mean(rej_POIS2),3)+mean(mean(rej_POIS2_2),3))/2;

% compute simualtion SD
Diff=0.5*rej_POIS2+0.5*rej_POIS2_2-rej_CLR;
V2=var(Diff,1);
Var2=sum(V2,3)/Lc^2;
SD2=sqrt(Var2/I);



Pow_diff=Rej_POIS2-Rej_CLR;
beta0_max=Beta0(Pow_diff==max(Pow_diff));

%implied correlation
Rho0=(Omega(1,2)-Beta0*Omega(2,2))./(Omega(1,1)-2*Beta0*Omega(1,2)+Beta0.^2*Omega(2,2)).^(0.5);


disp(' ')
fprintf('$\\rho='),fprintf('%8.2f $, \n',rho);
fprintf('$k='),fprintf('%8.0f $, \n',k);
fprintf('$max power diff='),fprintf('%.3f $, \n',max(Pow_diff));
fprintf('$beta0_max='),fprintf('%.3f $, \n',beta0_max);
fprintf('$avg power diff='),fprintf('%.3f $, \n',mean(Pow_diff));
fprintf('\\medskip{} \n');
disp(' ');
fprintf(' \\begin{tabular}{cccccc}\\hline \n');
fprintf('$\\beta_0$ &$\\rho_{uv,0}$ &       WAP2 &     WAP2-CLR &  SD & WAP2-AR\\\\ \\hline \n');

for j=1:Lg
    fprintf([sprintf('%.2f &',Beta0(j)) ...
    strrep(sprintf('%.2f & %.3f & %.3f & %.4f & %.3f\\\\\\\\ \n', ...
       Rho0(j), Rej_POIS2(j), Rej_POIS2(j)-Rej_CLR(j), SD2(j), Rej_POIS2(j)-Rej_AR(j)),'0.','.')]);
end

fprintf('\\hline \n')
fprintf('\\end{tabular}\n')
disp(' ')

beta0_max
[max(Pow_diff), mean(Pow_diff)]

%fname = sprintf('WAP_con_%d_%d.mat', k,rho*10);
%save(fname)
end
