% For Table III, Table IV and Table SM-V, high rho=.95 and.99
%Average (over \lambda) power difference WAP2-CLR, for high rho=.95 and.99
% this file complements the code WAP.m
%calls user-defined function CLR_pv.m, psi_h.m, psi2_h.m, psi_a.m,
%psi2_a.m.
clear all

I=1000; % number of replication
N=100; % sample size
Nsim=100000; %number of simulations when pvals are computed by simulations

rho=.95; %correlation between u and v, value considered: .95, .99.
k=5; % values of k considered k=[2,5,10,20,40]

Con=(2.5:2.5:90)'; %concentration paramter=Pi'Z'ZPi = lambda in the notes
Lc=length(Con);
beta_star=0;
omega_1=1;omega_2=1;
r_uv=rho/sqrt((1-rho^2));

Omega=[omega_1^2,rho;rho,omega_2^2];%reduced form covariance matrix
sigma_1=sqrt(omega_1^2-2*rho*beta_star+omega_2^2*beta_star^2); sigma_12=rho-omega_2^2*beta_star;sigma_2=omega_2;
Sigma=[sigma_1^2,sigma_12;sigma_12,sigma_2^2];

alpha=0.05;

%the null values
Beta0=[-10000, -1000,-100,-50,-10,-7.5,-5.0,-4:0.25:-0.25,0.25:0.25:4,5.0,7.5,10,50,100,1000,10000];
Beta0=Beta0+0.00001*(Beta0==1); %with beta_0=0, c_beta_2star=infinity
Beta0=Beta0+0.00001*(Beta0==2); %with beta_0=0, c_beta_2star=infinity for rho=0.5
Beta0=Beta0-0.00001*(Beta0==-1);

Lg=length(Beta0);

AR=zeros(I,Lg,Lc);
LR=zeros(I,Lg,Lc);
KLM=zeros(I,Lg,Lc);

POIS2=zeros(I,Lg,Lc);
POIS2_2=zeros(I,Lg,Lc);
p_AR=zeros(I,Lg,Lc);
p_CLR=zeros(I,Lg,Lc);
p_KLM=zeros(I,Lg,Lc);
p_POIS2=zeros(I,Lg,Lc);
p_POIS2_2=zeros(I,Lg,Lc);
rej_AR=zeros(I,Lg,Lc);
rej_CLR=zeros(I,Lg,Lc);
rej_KLM=zeros(I,Lg,Lc);
rej_POIS2=zeros(I,Lg,Lc);
rej_POIS2_2=zeros(I,Lg,Lc);


rng(0)
randseedoffset=81474;
tic
%for AMSPOIS2: C=c_beta, D=d_beta, Ksi=ksi_beta(Q); 
psi = @(C,D,Ksi,lambda) exp(-lambda.*(C.^2+D.^2)/2).*(lambda.*Ksi).^(-k/4+0.5)...
    .*besseli(k/2-1,sqrt(lambda.*Ksi));
psi2=@(D,q_t,lambda) exp(-lambda.*D.^2/2).*(lambda.*D.^2.*q_t).^(-k/4+0.5) ...
    .*besseli(k/2-1,sqrt(lambda.*D.^2.*q_t));
%---------------------------------
%for simulated p-vals
SS=normrnd(0,1,Nsim,k);
SQ_s=sum(SS.*SS,2);

parfor i=1:I
    rng(randseedoffset+i, 'twister'); %to generate the same random sequences
    ZZ = randn(N,k,Lc); %  instruments
    epsilon= randn(N,2,Lc);
    
    for j=1:Lg;
        
      beta0=Beta0(j);
      b_0=[1,-beta0]'; a_0=[beta0,1]';
      % compute the other point
      b=[1,-beta_star]';
      d_beta_star=(b'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
      c_beta_star=(beta_star-beta0)*(b_0'*Omega*b_0)^(-1/2);
        
      r_beta0=[1 0]/(Omega)*a_0*(a_0'/(Omega)*a_0)^(-1/2);
      d_beta0=(b_0'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
      beta_2star=beta0-d_beta0*(beta_star-beta0)/(d_beta0+2*r_beta0*(beta_star-beta0));
      Con2=Con*(d_beta0+2*r_beta0*(beta_star-beta0))^2/d_beta0^2;
      b2=[1,-beta_2star]';
      d_beta_2star=(b2'*Omega*b_0)*(b_0'*Omega*b_0)^(-1/2)*(det(Omega))^(-1/2);
      c_beta_2star=(beta_2star-beta0)*(b_0'*Omega*b_0)^(-1/2);
      S_POIS2=zeros(Nsim,Lc);
      for p=1:Lc
        con=Con(p);
        epsilon1=epsilon(:,1,p);
        epsilon2 = epsilon(:,2,p);
        Z=ZZ(:,:,p);
        U = epsilon1;
        V_2 = sigma_12*epsilon1 + sqrt(1-sigma_12^2)*epsilon2;
        Pi=ones(k,1)*sqrt(con/sum(sum(Z'*Z)));
        y_2 = Z*Pi + V_2;
        y_1 = y_2*beta_star+U;
        Y=[y_1,y_2]; V_1=U+V_2*beta_star;V=[V_1, V_2];
        
        S=(Z'*Z)^(-1/2)*Z'*Y*b_0/(b_0'*Omega*b_0)^(1/2);
        T=(Z'*Z)^(-1/2)*Z'*Y/Omega*a_0/(a_0'/Omega*a_0)^(1/2);
        Q_s=S'*S;
        Q_st=S'*T;
        Q_t=T'*T;
        S_2=Q_st/(Q_s*Q_t)^(1/2);
        
        AR(i,j,p)=Q_s;
        KLM(i,j,p)=Q_st^2/Q_t;
        LR(i,j,p)=0.5*(Q_s-Q_t+((Q_s+Q_t)^2-4*Q_s*Q_t*(1-S_2^2))^(1/2));
        % compute POIS2 
        %POIS2 under the data generated by (beta_star, con)
        ksi_beta_star=c_beta_star^2*Q_s+2*c_beta_star*d_beta_star*Q_st ...
            +d_beta_star^2*Q_t;
        ksi_beta_2star=c_beta_2star^2*Q_s+2*c_beta_2star*d_beta_2star*Q_st ...
            +d_beta_2star^2*Q_t;
        Check0=[sqrt(Con*ksi_beta_star),sqrt(Con2*ksi_beta_2star),sqrt(Con*d_beta_star^2*Q_t),sqrt(Con2*d_beta_2star^2*Q_t)]; 
        % the max value that would appear in the argument  in the besseli function of the first kind (in the deno or numerator)
        check0=max(Check0,[],2); % check0 determines whether the besseli function will be approximate, for each lambda
        sc1=sqrt(Con*ksi_beta_star)-Con*(c_beta_star^2+d_beta_star^2)/2;
        sc2=sqrt(Con2*ksi_beta_2star)-Con2*(c_beta_2star^2+d_beta_2star^2)/2;
        sc3=sqrt(Con*d_beta_star^2*Q_t)-Con*d_beta_star^2/2;
        sc4=sqrt(Con2*d_beta_2star^2*Q_t)-Con2*d_beta_2star^2/2;
        sc=max([sc1,sc2,sc3,sc4],[],2)*0.9;
        sc=max(sc,0); % do not re-scale if the component of sc is negative
        % exp(max(sc)) is a the rescaling term to prevent Inf in psi and psi2,
        % take maximum of four subjects
        POIS2(i,j,p)=(mean(psi_h(c_beta_star,d_beta_star,ksi_beta_star,Con,check0,k,sc)) ...
            +mean(psi_h(c_beta_2star,d_beta_2star,ksi_beta_2star,Con2,check0,k,sc)))...
            /mean(psi2_h(d_beta_star,Q_t,Con,k,check0,sc))/2;
        %POIS2 under the data generated by (beta_2star, con2)
        ksi_beta_star_2=c_beta_star^2*Q_s+2*c_beta_star*d_beta_star*(-Q_st) ...
            +d_beta_star^2*Q_t;
        ksi_beta_2star_2=c_beta_2star^2*Q_s+2*c_beta_2star*d_beta_2star*(-Q_st) ...
            +d_beta_2star^2*Q_t;
        Check02=[sqrt(Con*ksi_beta_star_2),sqrt(Con2*ksi_beta_2star_2),sqrt(Con*d_beta_star^2*Q_t),sqrt(Con2*d_beta_2star^2*Q_t)];
        check02=max(Check02,[],2);
        sc1_2=sqrt(Con*ksi_beta_star_2)-Con*(c_beta_star^2+d_beta_star^2)/2;
        sc2_2=sqrt(Con2*ksi_beta_2star_2)-Con2*(c_beta_2star^2+d_beta_2star^2)/2;
        sc_2=max([sc1_2,sc2_2,sc3,sc4],[],2)*0.9;
        sc_2=max(sc_2,0);
        POIS2_2(i,j,p)=(mean(psi_h(c_beta_star,d_beta_star,ksi_beta_star_2,Con,check02,k,sc_2)) ...
            +mean(psi_h(c_beta_2star,d_beta_2star,ksi_beta_2star_2,Con2,check02,k,sc_2)) )...
            /mean(psi2_h(d_beta_star,Q_t,Con,k,check02,sc_2))/2;
        %-------------------------------
        % p-values
        p_AR(i,j,p)=1-chi2cdf(AR(i,j,p),k);
        p_KLM(i,j,p)=1-chi2cdf(KLM(i,j,p),1);
        p_CLR(i,j,p)=CLR_pv(LR(i,j,p),Q_t,k); %by num integr
        % comptue rejection frequency
        rej_AR(i,j,p)=p_AR(i,j,p)<alpha;
        rej_KLM(i,j,p)=p_KLM(i,j,p)<alpha;
        rej_CLR(i,j,p)=p_CLR(i,j,p)<alpha;
        %for simulated p-vals
        SQ_st=SS*T;
        %simulated p-vals for AMSPOIS2------------------------
        S_POIS2_nu=zeros(Nsim,Lc);
        S_POIS2_de=zeros(Nsim,Lc);
        
        M_S_ksi_beta_star=max(c_beta_star^2*SQ_s)+2*max(c_beta_star*d_beta_star*SQ_st)+d_beta_star^2*Q_t;
        M_S_ksi_beta_2star=max(c_beta_2star^2*SQ_s)+2*max(c_beta_2star*d_beta_2star*SQ_st)+d_beta_2star^2*Q_t;
        Check0=[sqrt(Con*M_S_ksi_beta_star),sqrt(Con2*M_S_ksi_beta_2star),sqrt(Con*d_beta_star^2*Q_t)]; 
        S_check=max(Check0,[],2);
        S_sc1=sqrt(Con*M_S_ksi_beta_star)-Con*(c_beta_star^2+d_beta_star^2)/2;
        S_sc2=sqrt(Con2*M_S_ksi_beta_2star)-Con2*(c_beta_2star^2+d_beta_2star^2)/2;
        S_sc3=sqrt(Con*d_beta_star^2*Q_t)-Con*d_beta_star^2/2;
        S_sc=max([S_sc1,S_sc2,S_sc3],[],2)*0.9; 
        S_sc=max(S_sc,0); % if the re-scaling factor exp(S_sc) is smaller than one, then do not re-scale
        for t=1:Lc
        S_ksi_beta_star=c_beta_star^2*SQ_s+2*c_beta_star*d_beta_star*SQ_st ...
            +d_beta_star^2*Q_t;
        S_ksi_beta_2star=c_beta_2star^2*SQ_s+2*c_beta_2star*d_beta_2star*SQ_st ...
            +d_beta_2star^2*Q_t;
        S_POIS2_nu(:,t)=(psi_a(c_beta_star,d_beta_star,S_ksi_beta_star,Con(t),S_check,k, S_sc)...
            +psi_a(c_beta_2star,d_beta_2star,S_ksi_beta_2star,Con2(t),S_check,k, S_sc));
        S_POIS2_de(:,t)=psi2_a(d_beta_star,Q_t,Con(t),k, S_check,S_sc);
        end
        S_POIS2(:,p)=mean(S_POIS2_nu,2)./mean(S_POIS2_de,2)/2;
        p_POIS2(i,j,p)=1-sum(S_POIS2(:,p)<POIS2(i,j,p))/Nsim; % under the data generated by (beta_star, con)
        rej_POIS2(i,j,p)=p_POIS2(i,j,p)<alpha;
        p_POIS2_2(i,j,p)=1-sum(S_POIS2(:,p)<POIS2_2(i,j,p))/Nsim; % under the data generated by (beta_2star, con2)
        rej_POIS2_2(i,j,p)=p_POIS2_2(i,j,p)<alpha; 
        %------------------------------------------------------------
        
      end
        
    end
    
end
toc
Rej_AR=mean(mean(rej_AR),3);
Rej_KLM=mean(mean(rej_KLM),3);
Rej_CLR=mean(mean(rej_CLR),3);
Rej_POIS2=(mean(mean(rej_POIS2),3)+mean(mean(rej_POIS2_2),3))/2;

% compute simualtion SD
Diff=0.5*rej_POIS2+0.5*rej_POIS2_2-rej_CLR;
V2=var(Diff,1);
Var2=sum(V2,3)/Lc^2;
SD2=sqrt(Var2/I);


Pow_diff=Rej_POIS2-Rej_CLR;
beta0_max=Beta0(Pow_diff==max(Pow_diff));

%implied correlation
Rho0=(Omega(1,2)-Beta0*Omega(2,2))./(Omega(1,1)-2*Beta0*Omega(1,2)+Beta0.^2*Omega(2,2)).^(0.5);


disp(' ')
fprintf('$\\rho='),fprintf('%8.2f $, \n',rho);
fprintf('$k='),fprintf('%8.0f $, \n',k);
fprintf('$max power diff='),fprintf('%.3f $, \n',max(Pow_diff));
fprintf('$beta0_max='),fprintf('%.3f $, \n',beta0_max);
fprintf('$avg power diff='),fprintf('%.3f $, \n',mean(Pow_diff));
fprintf('\\medskip{} \n');
disp(' ');
fprintf(' \\begin{tabular}{cccccc}\\hline \n');
fprintf('$\\beta_0$ &$\\rho_{uv,0}$ &       WAP2 &     WAP2-CLR &  SD & WAP2-AR\\\\ \\hline \n');


for j=1:Lg
    fprintf([sprintf('%.2f &',Beta0(j)) ...
    strrep(sprintf('%.2f & %.3f & %.3f & %.4f & %.3f\\\\\\\\ \n', ...
       Rho0(j), Rej_POIS2(j), Rej_POIS2(j)-Rej_CLR(j), SD2(j), Rej_POIS2(j)-Rej_AR(j)),'0.','.')]);
end

fprintf('\\hline \n')
fprintf('\\end{tabular}\n')
disp(' ')
%fname = sprintf('WAP_con_%d_%d.mat', k,rho*10);
%save(fname)
%end